@@ -120,9 +120,7 @@ use std::str;
 use std::time::{self, Duration};
 use std::usize;
 
-use cargo;
 use cargo::util::{is_ci, CargoResult, ProcessBuilder, ProcessError, Rustc};
-use filetime;
 use serde_json::{self, Value};
 use url::Url;
 
@@ -1645,6 +1643,7 @@ fn substitute_macros(input: &str) -> String {
         ("[FINISHED]", "    Finished"),
         ("[ERROR]", "error:"),
         ("[WARNING]", "warning:"),
+        ("[NOTE]", "note:"),
         ("[DOCUMENTING]", " Documenting"),
         ("[FRESH]", "       Fresh"),
         ("[UPDATING]", "    Updating"),
@@ -1666,7 +1665,6 @@ fn substitute_macros(input: &str) -> String {
         ("[IGNORED]", "     Ignored"),
         ("[INSTALLED]", "   Installed"),
         ("[REPLACED]", "    Replaced"),
-        ("[NOTE]", "        Note"),
     ];
     let mut result = input.to_owned();
     for &(pat, subst) in &macros {
@@ -1805,9 +1803,9 @@ pub fn slow_cpu_multiplier(main: u64) -> Duration {
     Duration::from_secs(*SLOW_CPU_MULTIPLIER * main)
 }
 
-pub fn clippy_is_available() -> bool {
-    if let Err(e) = process("clippy-driver").arg("-V").exec_with_output() {
-        eprintln!("clippy-driver not available, skipping clippy test");
+pub fn command_is_available(cmd: &str) -> bool {
+    if let Err(e) = process(cmd).arg("-V").exec_with_output() {
+        eprintln!("{} not available, skipping tests", cmd);
         eprintln!("{:?}", e);
         false
     } else {
