@@ -3,6 +3,7 @@
 //! This module implements some simple tracking information for timing of how
 //! long it takes for different units to compile.
 use super::{CompileMode, Unit};
+use crate::core::compiler::job_queue::JobId;
 use crate::core::compiler::BuildContext;
 use crate::core::PackageId;
 use crate::util::cpu::State;
@@ -41,7 +42,7 @@ pub struct Timings<'a, 'cfg> {
     unit_times: Vec<UnitTime<'a>>,
     /// Units that are in the process of being built.
     /// When they finished, they are moved to `unit_times`.
-    active: HashMap<u32, UnitTime<'a>>,
+    active: HashMap<JobId, UnitTime<'a>>,
     /// Concurrency-tracking information. This is periodically updated while
     /// compilation progresses.
     concurrency: Vec<Concurrency>,
@@ -84,6 +85,10 @@ struct Concurrency {
     /// Number of units that are not yet ready, because they are waiting for
     /// dependencies to finish.
     inactive: usize,
+    /// Number of rustc "extra" threads -- i.e., how many tokens have been
+    /// provided across all current rustc instances that are not the main thread
+    /// tokens.
+    rustc_parallelism: usize,
 }
 
 impl<'a, 'cfg> Timings<'a, 'cfg> {
@@ -140,7 +145,7 @@ impl<'a, 'cfg> Timings<'a, 'cfg> {
     }
 
     /// Mark that a unit has started running.
-    pub fn unit_start(&mut self, id: u32, unit: Unit<'a>) {
+    pub fn unit_start(&mut self, id: JobId, unit: Unit<'a>) {
         if !self.enabled {
             return;
         }
@@ -174,7 +179,7 @@ impl<'a, 'cfg> Timings<'a, 'cfg> {
     }
 
     /// Mark that the `.rmeta` file as generated.
-    pub fn unit_rmeta_finished(&mut self, id: u32, unlocked: Vec<&Unit<'a>>) {
+    pub fn unit_rmeta_finished(&mut self, id: JobId, unlocked: Vec<&Unit<'a>>) {
         if !self.enabled {
             return;
         }
@@ -192,7 +197,7 @@ impl<'a, 'cfg> Timings<'a, 'cfg> {
     }
 
     /// Mark that a unit has finished running.
-    pub fn unit_finished(&mut self, id: u32, unlocked: Vec<&Unit<'a>>) {
+    pub fn unit_finished(&mut self, id: JobId, unlocked: Vec<&Unit<'a>>) {
         if !self.enabled {
             return;
         }
@@ -232,7 +237,13 @@ impl<'a, 'cfg> Timings<'a, 'cfg> {
     }
 
     /// This is called periodically to mark the concurrency of internal structures.
-    pub fn mark_concurrency(&mut self, active: usize, waiting: usize, inactive: usize) {
+    pub fn mark_concurrency(
+        &mut self,
+        active: usize,
+        waiting: usize,
+        inactive: usize,
+        rustc_parallelism: usize,
+    ) {
         if !self.enabled {
             return;
         }
@@ -241,6 +252,7 @@ impl<'a, 'cfg> Timings<'a, 'cfg> {
             active,
             waiting,
             inactive,
+            rustc_parallelism,
         };
         self.concurrency.push(c);
     }
@@ -281,21 +293,29 @@ impl<'a, 'cfg> Timings<'a, 'cfg> {
     }
 
     /// Call this when all units are finished.
-    pub fn finished(&mut self, bcx: &BuildContext<'_, '_>) -> CargoResult<()> {
+    pub fn finished(
+        &mut self,
+        bcx: &BuildContext<'_, '_>,
+        error: &Option<anyhow::Error>,
+    ) -> CargoResult<()> {
         if !self.enabled {
             return Ok(());
         }
-        self.mark_concurrency(0, 0, 0);
+        self.mark_concurrency(0, 0, 0, 0);
         self.unit_times
             .sort_unstable_by(|a, b| a.start.partial_cmp(&b.start).unwrap());
         if self.report_html {
-            self.report_html(bcx)?;
+            self.report_html(bcx, error)?;
         }
         Ok(())
     }
 
     /// Save HTML report to disk.
-    fn report_html(&self, bcx: &BuildContext<'_, '_>) -> CargoResult<()> {
+    fn report_html(
+        &self,
+        bcx: &BuildContext<'_, '_>,
+        error: &Option<anyhow::Error>,
+    ) -> CargoResult<()> {
         let duration = d_as_f64(self.start.elapsed());
         let timestamp = self.start_str.replace(&['-', ':'][..], "");
         let filename = format!("cargo-timing-{}.html", timestamp);
@@ -306,7 +326,7 @@ impl<'a, 'cfg> Timings<'a, 'cfg> {
             .map(|(name, _targets)| name.as_str())
             .collect();
         f.write_all(HTML_TMPL.replace("{ROOTS}", &roots.join(", ")).as_bytes())?;
-        self.write_summary_table(&mut f, duration, bcx)?;
+        self.write_summary_table(&mut f, duration, bcx, error)?;
         f.write_all(HTML_CANVAS.as_bytes())?;
         self.write_unit_table(&mut f)?;
         // It helps with pixel alignment to use whole numbers.
@@ -347,6 +367,7 @@ impl<'a, 'cfg> Timings<'a, 'cfg> {
         f: &mut impl Write,
         duration: f64,
         bcx: &BuildContext<'_, '_>,
+        error: &Option<anyhow::Error>,
     ) -> CargoResult<()> {
         let targets: Vec<String> = self
             .root_targets
@@ -361,7 +382,24 @@ impl<'a, 'cfg> Timings<'a, 'cfg> {
         };
         let total_time = format!("{:.1}s{}", duration, time_human);
         let max_concurrency = self.concurrency.iter().map(|c| c.active).max().unwrap();
+        let max_rustc_concurrency = self
+            .concurrency
+            .iter()
+            .map(|c| c.rustc_parallelism)
+            .max()
+            .unwrap();
         let rustc_info = render_rustc_info(bcx);
+        let error_msg = match error {
+            Some(e) => format!(
+                r#"\
+  <tr>
+    <td class="error-text">Error:</td><td>{}</td>
+  </tr>
+"#,
+                e
+            ),
+            None => "".to_string(),
+        };
         write!(
             f,
             r#"
@@ -393,7 +431,10 @@ impl<'a, 'cfg> Timings<'a, 'cfg> {
   <tr>
     <td>rustc:</td><td>{}</td>
   </tr>
-
+  <tr>
+    <td>Max (global) rustc threads concurrency:</td><td>{}</td>
+  </tr>
+{}
 </table>
 "#,
             targets,
@@ -407,6 +448,8 @@ impl<'a, 'cfg> Timings<'a, 'cfg> {
             self.start_str,
             total_time,
             rustc_info,
+            max_rustc_concurrency,
+            error_msg,
         )?;
         Ok(())
     }
@@ -562,15 +605,17 @@ fn d_as_f64(d: Duration) -> f64 {
 
 fn render_rustc_info(bcx: &BuildContext<'_, '_>) -> String {
     let version = bcx
-        .rustc
+        .rustc()
         .verbose_version
         .lines()
         .next()
         .expect("rustc version");
-    let requested_target = bcx.build_config.requested_kind.short_name(bcx);
+    let requested_target = bcx.target_data.short_name(&bcx.build_config.requested_kind);
     format!(
         "{}<br>Host: {}<br>Target: {}",
-        version, bcx.rustc.host, requested_target
+        version,
+        bcx.rustc().host,
+        requested_target
     )
 }
 
@@ -676,6 +721,10 @@ h1 {
   text-align: center;
 }
 
+.error-text {
+  color: #e80000;
+}
+
 </style>
 </head>
 <body>
