@@ -31,10 +31,11 @@ use std::sync::Arc;
 use crate::core::compiler::standard_lib;
 use crate::core::compiler::unit_dependencies::build_unit_dependencies;
 use crate::core::compiler::{BuildConfig, BuildContext, Compilation, Context};
-use crate::core::compiler::{CompileKind, CompileMode, Unit};
+use crate::core::compiler::{CompileKind, CompileMode, RustcTargetData, Unit};
 use crate::core::compiler::{DefaultExecutor, Executor, UnitInterner};
 use crate::core::profiles::{Profiles, UnitFor};
-use crate::core::resolver::{Resolve, ResolveOpts};
+use crate::core::resolver::features::{self, FeaturesFor};
+use crate::core::resolver::{HasDevUnits, Resolve, ResolveOpts};
 use crate::core::{LibKind, Package, PackageSet, Target};
 use crate::core::{PackageId, PackageIdSpec, TargetKind, Workspace};
 use crate::ops;
@@ -111,7 +112,7 @@ impl Packages {
         Ok(match (all, exclude.len(), package.len()) {
             (false, 0, 0) => Packages::Default,
             (false, 0, _) => Packages::Packages(package),
-            (false, _, _) => failure::bail!("--exclude can only be used together with --workspace"),
+            (false, _, _) => anyhow::bail!("--exclude can only be used together with --workspace"),
             (true, 0, _) => Packages::All,
             (true, _, _) => Packages::OptOut(exclude),
         })
@@ -160,13 +161,13 @@ impl Packages {
         };
         if specs.is_empty() {
             if ws.is_virtual() {
-                failure::bail!(
+                anyhow::bail!(
                     "manifest path `{}` contains no package: The manifest is virtual, \
                      and the workspace has no members.",
                     ws.root().display()
                 )
             }
-            failure::bail!("no packages to compile")
+            anyhow::bail!("no packages to compile")
         }
         Ok(specs)
     }
@@ -185,7 +186,7 @@ impl Packages {
                     ws.members()
                         .find(|pkg| pkg.name().as_str() == name)
                         .ok_or_else(|| {
-                            failure::format_err!(
+                            anyhow::format_err!(
                                 "package `{}` is not a member of the workspace",
                                 name
                             )
@@ -300,22 +301,38 @@ pub fn compile_ws<'a>(
         }
     }
 
-    let profiles = ws.profiles();
-
-    // Early check for whether the profile is defined.
-    let _ = profiles.base_profile(&build_config.profile_kind)?;
+    let profiles = Profiles::new(
+        ws.profiles(),
+        config,
+        build_config.requested_profile,
+        ws.features(),
+    )?;
+    let target_data = RustcTargetData::new(ws, build_config.requested_kind)?;
 
     let specs = spec.to_package_id_specs(ws)?;
     let dev_deps = ws.require_optional_deps() || filter.need_dev_deps(build_config.mode);
     let opts = ResolveOpts::new(dev_deps, features, all_features, !no_default_features);
-    let resolve = ops::resolve_ws_with_opts(ws, opts, &specs)?;
+    let has_dev_units = if filter.need_dev_deps(build_config.mode) {
+        HasDevUnits::Yes
+    } else {
+        HasDevUnits::No
+    };
+    let resolve = ops::resolve_ws_with_opts(
+        ws,
+        &target_data,
+        build_config.requested_kind,
+        &opts,
+        &specs,
+        has_dev_units,
+    )?;
     let WorkspaceResolve {
         mut pkg_set,
         workspace_resolve,
         targeted_resolve: resolve,
+        resolved_features,
     } = resolve;
 
-    let std_resolve = if let Some(crates) = &config.cli_unstable().build_std {
+    let std_resolve_features = if let Some(crates) = &config.cli_unstable().build_std {
         if build_config.build_plan {
             config
                 .shell()
@@ -325,12 +342,13 @@ pub fn compile_ws<'a>(
             // TODO: This should eventually be fixed. Unfortunately it is not
             // easy to get the host triple in BuildConfig. Consider changing
             // requested_target to an enum, or some other approach.
-            failure::bail!("-Zbuild-std requires --target");
+            anyhow::bail!("-Zbuild-std requires --target");
         }
-        let (mut std_package_set, std_resolve) = standard_lib::resolve_std(ws, crates)?;
+        let (mut std_package_set, std_resolve, std_features) =
+            standard_lib::resolve_std(ws, &target_data, build_config.requested_kind, crates)?;
         remove_dylib_crate_type(&mut std_package_set)?;
         pkg_set.add_set(std_package_set);
-        Some(std_resolve)
+        Some((std_resolve, std_features))
     } else {
         None
     };
@@ -359,7 +377,7 @@ pub fn compile_ws<'a>(
             && !ws.is_member(pkg)
             && pkg.dependencies().iter().any(|dep| !dep.is_transitive())
         {
-            failure::bail!(
+            anyhow::bail!(
                 "package `{}` cannot be tested because it requires dev-dependencies \
                  and is not a member of the workspace",
                 pkg.name()
@@ -381,6 +399,7 @@ pub fn compile_ws<'a>(
     }
 
     profiles.validate_packages(
+        ws.profiles(),
         &mut config.shell(),
         workspace_resolve.as_ref().unwrap_or(&resolve),
     )?;
@@ -394,14 +413,16 @@ pub fn compile_ws<'a>(
         profiles,
         &interner,
         HashMap::new(),
+        target_data,
     )?;
+
     let units = generate_targets(
         ws,
-        profiles,
         &to_builds,
         filter,
         build_config.requested_kind,
         &resolve,
+        &resolved_features,
         &bcx,
     )?;
 
@@ -418,10 +439,12 @@ pub fn compile_ws<'a>(
                 crates.push("test".to_string());
             }
         }
+        let (std_resolve, std_features) = std_resolve_features.as_ref().unwrap();
         standard_lib::generate_std_roots(
             &bcx,
             &crates,
-            std_resolve.as_ref().unwrap(),
+            std_resolve,
+            std_features,
             build_config.requested_kind,
         )?
     } else {
@@ -430,7 +453,7 @@ pub fn compile_ws<'a>(
 
     if let Some(args) = extra_args {
         if units.len() != 1 {
-            failure::bail!(
+            anyhow::bail!(
                 "extra arguments to `{}` can only be passed to one \
                  target, consider filtering\nthe package by passing, \
                  e.g., `--lib` or `--bin NAME` to specify a single target",
@@ -447,7 +470,7 @@ pub fn compile_ws<'a>(
             // the target is a binary. Binary crates get their private items
             // documented by default.
             if rustdoc_document_private_items || unit.target.is_bin() {
-                let mut args = extra_args.take().unwrap_or(vec![]);
+                let mut args = extra_args.take().unwrap_or_else(|| vec![]);
                 args.push("--document-private-items".into());
                 extra_args = Some(args);
             }
@@ -458,8 +481,14 @@ pub fn compile_ws<'a>(
         }
     }
 
-    let unit_dependencies =
-        build_unit_dependencies(&bcx, &resolve, std_resolve.as_ref(), &units, &std_roots)?;
+    let unit_dependencies = build_unit_dependencies(
+        &bcx,
+        &resolve,
+        &resolved_features,
+        std_resolve_features.as_ref(),
+        &units,
+        &std_roots,
+    )?;
 
     let ret = {
         let _p = profile::start("compiling");
@@ -652,11 +681,11 @@ struct Proposal<'a> {
 /// compile. Dependencies for these targets are computed later in `unit_dependencies`.
 fn generate_targets<'a>(
     ws: &Workspace<'_>,
-    profiles: &Profiles,
     packages: &[&'a Package],
     filter: &CompileFilter,
     default_arch_kind: CompileKind,
     resolve: &'a Resolve,
+    resolved_features: &features::ResolvedFeatures,
     bcx: &BuildContext<'a, '_>,
 ) -> CargoResult<Vec<Unit<'a>>> {
     // Helper for creating a `Unit` struct.
@@ -716,14 +745,14 @@ fn generate_targets<'a>(
             _ => target_mode,
         };
         let kind = default_arch_kind.for_target(target);
-        let profile = profiles.get_profile(
+        let profile =
+            bcx.profiles
+                .get_profile(pkg.package_id(), ws.is_member(pkg), unit_for, target_mode);
+
+        let features = Vec::from(resolved_features.activated_features(
             pkg.package_id(),
-            ws.is_member(pkg),
-            unit_for,
-            target_mode,
-            bcx.build_config.profile_kind.clone(),
-        );
-        let features = resolve.features_sorted(pkg.package_id());
+            FeaturesFor::NormalOrDev, // Root units are never build dependencies.
+        ));
         bcx.units.intern(
             pkg,
             target,
@@ -793,12 +822,9 @@ fn generate_targets<'a>(
                 if !all_targets && libs.is_empty() && *lib == LibRule::True {
                     let names = packages.iter().map(|pkg| pkg.name()).collect::<Vec<_>>();
                     if names.len() == 1 {
-                        failure::bail!("no library targets found in package `{}`", names[0]);
+                        anyhow::bail!("no library targets found in package `{}`", names[0]);
                     } else {
-                        failure::bail!(
-                            "no library targets found in packages: {}",
-                            names.join(", ")
-                        );
+                        anyhow::bail!("no library targets found in packages: {}", names.join(", "));
                     }
                 }
                 proposals.extend(libs);
@@ -860,6 +886,10 @@ fn generate_targets<'a>(
 
     // Only include targets that are libraries or have all required
     // features available.
+    //
+    // `features_map` is a map of &Package -> enabled_features
+    // It is computed by the set of enabled features for the package plus
+    // every enabled feature of every enabled dependency.
     let mut features_map = HashMap::new();
     let mut units = HashSet::new();
     for Proposal {
@@ -871,9 +901,9 @@ fn generate_targets<'a>(
     {
         let unavailable_features = match target.required_features() {
             Some(rf) => {
-                let features = features_map
-                    .entry(pkg)
-                    .or_insert_with(|| resolve_all_features(resolve, pkg.package_id()));
+                let features = features_map.entry(pkg).or_insert_with(|| {
+                    resolve_all_features(resolve, resolved_features, pkg.package_id())
+                });
                 rf.iter().filter(|f| !features.contains(*f)).collect()
             }
             None => Vec::new(),
@@ -887,7 +917,7 @@ fn generate_targets<'a>(
                 .iter()
                 .map(|s| format!("`{}`", s))
                 .collect();
-            failure::bail!(
+            anyhow::bail!(
                 "target `{}` in package `{}` requires the features: {}\n\
                  Consider enabling them by passing, e.g., `--features=\"{}\"`",
                 target.name(),
@@ -901,17 +931,34 @@ fn generate_targets<'a>(
     Ok(units.into_iter().collect())
 }
 
+/// Gets all of the features enabled for a package, plus its dependencies'
+/// features.
+///
+/// Dependencies are added as `dep_name/feat_name` because `required-features`
+/// wants to support that syntax.
 fn resolve_all_features(
     resolve_with_overrides: &Resolve,
+    resolved_features: &features::ResolvedFeatures,
     package_id: PackageId,
 ) -> HashSet<String> {
-    let mut features = resolve_with_overrides.features(package_id).clone();
+    let mut features: HashSet<String> = resolved_features
+        .activated_features(package_id, FeaturesFor::NormalOrDev)
+        .iter()
+        .map(|s| s.to_string())
+        .collect();
 
     // Include features enabled for use by dependencies so targets can also use them with the
     // required-features field when deciding whether to be built or skipped.
-    for (dep, _) in resolve_with_overrides.deps(package_id) {
-        for feature in resolve_with_overrides.features(dep) {
-            features.insert(dep.name().to_string() + "/" + feature);
+    for (dep_id, deps) in resolve_with_overrides.deps(package_id) {
+        for dep in deps {
+            let features_for = if dep.is_build() {
+                FeaturesFor::BuildDep
+            } else {
+                FeaturesFor::NormalOrDev
+            };
+            for feature in resolved_features.activated_features(dep_id, features_for) {
+                features.insert(dep.name_in_toml().to_string() + "/" + &feature);
+            }
         }
     }
 
@@ -991,7 +1038,7 @@ fn find_named_targets<'a>(
                 .filter(|target| is_expected_kind(target))
         });
         let suggestion = closest_msg(target_name, targets, |t| t.name());
-        failure::bail!(
+        anyhow::bail!(
             "no {} target named `{}`{}",
             target_desc,
             target_name,
