@@ -24,7 +24,7 @@ pub fn generate_lockfile(ws: &Workspace<'_>) -> CargoResult<()> {
     let resolve = ops::resolve_with_previous(
         &mut registry,
         ws,
-        ResolveOpts::everything(),
+        &ResolveOpts::everything(),
         None,
         None,
         &[],
@@ -36,15 +36,15 @@ pub fn generate_lockfile(ws: &Workspace<'_>) -> CargoResult<()> {
 
 pub fn update_lockfile(ws: &Workspace<'_>, opts: &UpdateOptions<'_>) -> CargoResult<()> {
     if opts.aggressive && opts.precise.is_some() {
-        failure::bail!("cannot specify both aggressive and precise simultaneously")
+        anyhow::bail!("cannot specify both aggressive and precise simultaneously")
     }
 
     if ws.members().count() == 0 {
-        failure::bail!("you can't generate a lockfile for an empty workspace.")
+        anyhow::bail!("you can't generate a lockfile for an empty workspace.")
     }
 
     if opts.config.offline() {
-        failure::bail!("you can't update in the offline mode");
+        anyhow::bail!("you can't update in the offline mode");
     }
 
     // Updates often require a lot of modifications to the registry, so ensure
@@ -64,7 +64,7 @@ pub fn update_lockfile(ws: &Workspace<'_>, opts: &UpdateOptions<'_>) -> CargoRes
                     ops::resolve_with_previous(
                         &mut registry,
                         ws,
-                        ResolveOpts::everything(),
+                        &ResolveOpts::everything(),
                         None,
                         None,
                         &[],
@@ -110,7 +110,7 @@ pub fn update_lockfile(ws: &Workspace<'_>, opts: &UpdateOptions<'_>) -> CargoRes
     let resolve = ops::resolve_with_previous(
         &mut registry,
         ws,
-        ResolveOpts::everything(),
+        &ResolveOpts::everything(),
         Some(&previous_resolve),
         Some(&to_avoid),
         &[],
