@@ -67,7 +67,10 @@ impl<'cfg> PathSource<'cfg> {
 
         match self.packages.iter().find(|p| p.root() == &*self.path) {
             Some(pkg) => Ok(pkg.clone()),
-            None => Err(internal("no package found in source")),
+            None => Err(internal(format!(
+                "no package found in source {:?}",
+                self.path
+            ))),
         }
     }
 
@@ -147,6 +150,8 @@ impl<'cfg> PathSource<'cfg> {
             if let Some(result) = self.discover_git_and_list_files(pkg, root, &mut filter) {
                 return result;
             }
+            // no include option and not git repo discovered (see rust-lang/cargo#7183).
+            return self.list_files_walk_except_dot_files_and_dirs(pkg, &mut filter);
         }
         self.list_files_walk(pkg, &mut filter)
     }
@@ -206,7 +211,7 @@ impl<'cfg> PathSource<'cfg> {
         let index = repo.index()?;
         let root = repo
             .workdir()
-            .ok_or_else(|| internal("Can't list files on a bare repository."))?;
+            .ok_or_else(|| anyhow::format_err!("can't list files on a bare repository"))?;
         let pkg_path = pkg.root();
 
         let mut ret = Vec::<PathBuf>::new();
@@ -288,7 +293,7 @@ impl<'cfg> PathSource<'cfg> {
                 warn!("  found submodule {}", file_path.display());
                 let rel = file_path.strip_prefix(root)?;
                 let rel = rel.to_str().ok_or_else(|| {
-                    failure::format_err!("invalid utf-8 filename: {}", rel.display())
+                    anyhow::format_err!("invalid utf-8 filename: {}", rel.display())
                 })?;
                 // Git submodules are currently only named through `/` path
                 // separators, explicitly not `\` which windows uses. Who knew?
@@ -321,14 +326,37 @@ impl<'cfg> PathSource<'cfg> {
             use std::str;
             match str::from_utf8(data) {
                 Ok(s) => Ok(path.join(s)),
-                Err(..) => Err(internal(
-                    "cannot process path in git with a non \
-                     unicode filename",
+                Err(e) => Err(anyhow::format_err!(
+                    "cannot process path in git with a non utf8 filename: {}\n{:?}",
+                    e,
+                    data
                 )),
             }
         }
     }
 
+    fn list_files_walk_except_dot_files_and_dirs(
+        &self,
+        pkg: &Package,
+        filter: &mut dyn FnMut(&Path) -> CargoResult<bool>,
+    ) -> CargoResult<Vec<PathBuf>> {
+        let root = pkg.root();
+        let mut exclude_dot_files_dir_builder = GitignoreBuilder::new(root);
+        exclude_dot_files_dir_builder.add_line(None, ".*")?;
+        let ignore_dot_files_and_dirs = exclude_dot_files_dir_builder.build()?;
+
+        let mut filter_ignore_dot_files_and_dirs = |path: &Path| -> CargoResult<bool> {
+            let relative_path = path.strip_prefix(root)?;
+            match ignore_dot_files_and_dirs
+                .matched_path_or_any_parents(relative_path, /* is_dir */ false)
+            {
+                Match::Ignore(_) => Ok(false),
+                _ => filter(path),
+            }
+        };
+        self.list_files_walk(pkg, &mut filter_ignore_dot_files_and_dirs)
+    }
+
     fn list_files_walk(
         &self,
         pkg: &Package,
@@ -368,10 +396,6 @@ impl<'cfg> PathSource<'cfg> {
         entries.sort_unstable_by(|a, b| a.as_os_str().cmp(b.as_os_str()));
         for path in entries {
             let name = path.file_name().and_then(|s| s.to_str());
-            // Skip dotfile directories.
-            if name.map(|s| s.starts_with('.')) == Some(true) {
-                continue;
-            }
             if is_root && name == Some("target") {
                 // Skip Cargo artifacts.
                 continue;
@@ -383,7 +407,10 @@ impl<'cfg> PathSource<'cfg> {
 
     pub fn last_modified_file(&self, pkg: &Package) -> CargoResult<(FileTime, PathBuf)> {
         if !self.updated {
-            return Err(internal("BUG: source was not updated"));
+            return Err(internal(format!(
+                "BUG: source `{:?}` was not updated",
+                self.path
+            )));
         }
 
         let mut max = FileTime::zero();
