@@ -4,16 +4,22 @@ use crate::util::config::value;
 use crate::util::config::{Config, ConfigError, ConfigKey};
 use crate::util::config::{ConfigValue as CV, Definition, Value};
 use serde::{de, de::IntoDeserializer};
-use std::collections::HashSet;
-use std::path::PathBuf;
 use std::vec;
 
 /// Serde deserializer used to convert config values to a target type using
 /// `Config::get`.
 #[derive(Clone)]
-pub(crate) struct Deserializer<'config> {
-    pub(crate) config: &'config Config,
-    pub(crate) key: ConfigKey,
+pub(super) struct Deserializer<'config> {
+    pub(super) config: &'config Config,
+    /// The current key being deserialized.
+    pub(super) key: ConfigKey,
+    /// Whether or not this key part is allowed to be an inner table. For
+    /// example, `profile.dev.build-override` needs to check if
+    /// CARGO_PROFILE_DEV_BUILD_OVERRIDE_ prefixes exist. But
+    /// CARGO_BUILD_TARGET should not check for prefixes because it would
+    /// collide with CARGO_BUILD_TARGET_DIR. See `ConfigMapAccess` for
+    /// details.
+    pub(super) env_prefix_ok: bool,
 }
 
 macro_rules! deserialize_method {
@@ -22,13 +28,15 @@ macro_rules! deserialize_method {
         where
             V: de::Visitor<'de>,
         {
-            let v = self.config.$getter(&self.key)?.ok_or_else(||
-                ConfigError::missing(&self.key))?;
-            let Value{val, definition} = v;
+            let v = self
+                .config
+                .$getter(&self.key)?
+                .ok_or_else(|| ConfigError::missing(&self.key))?;
+            let Value { val, definition } = v;
             let res: Result<V::Value, ConfigError> = visitor.$visit(val);
             res.map_err(|e| e.with_key_context(&self.key, definition))
         }
-    }
+    };
 }
 
 impl<'de, 'config> de::Deserializer<'de> for Deserializer<'config> {
@@ -38,49 +46,64 @@ impl<'de, 'config> de::Deserializer<'de> for Deserializer<'config> {
     where
         V: de::Visitor<'de>,
     {
-        // Future note: If you ever need to deserialize a non-self describing
-        // map type, this should implement a starts_with check (similar to how
-        // ConfigMapAccess does).
-        if let Some(v) = self.config.env.get(self.key.as_env_key()) {
-            let res: Result<V::Value, ConfigError> = if v == "true" || v == "false" {
-                visitor.visit_bool(v.parse().unwrap())
-            } else if let Ok(v) = v.parse::<i64>() {
-                visitor.visit_i64(v)
+        // Determine if value comes from env, cli, or file, and merge env if
+        // possible.
+        let cv = self.config.get_cv(&self.key)?;
+        let env = self.config.env.get(self.key.as_env_key());
+        let env_def = Definition::Environment(self.key.as_env_key().to_string());
+        let use_env = match (&cv, env) {
+            (Some(cv), Some(_)) => env_def.is_higher_priority(cv.definition()),
+            (None, Some(_)) => true,
+            _ => false,
+        };
+        if use_env {
+            // Future note: If you ever need to deserialize a non-self describing
+            // map type, this should implement a starts_with check (similar to how
+            // ConfigMapAccess does).
+            let env = env.unwrap();
+            let res: Result<V::Value, ConfigError> = if env == "true" || env == "false" {
+                visitor.visit_bool(env.parse().unwrap())
+            } else if let Ok(env) = env.parse::<i64>() {
+                visitor.visit_i64(env)
             } else if self.config.cli_unstable().advanced_env
-                && v.starts_with('[')
-                && v.ends_with(']')
+                && env.starts_with('[')
+                && env.ends_with(']')
             {
                 visitor.visit_seq(ConfigSeqAccess::new(self.clone())?)
             } else {
-                visitor.visit_string(v.clone())
+                // Try to merge if possible.
+                match cv {
+                    Some(CV::List(_cv_list, _cv_def)) => {
+                        visitor.visit_seq(ConfigSeqAccess::new(self.clone())?)
+                    }
+                    _ => {
+                        // Note: CV::Table merging is not implemented, as env
+                        // vars do not support table values.
+                        visitor.visit_str(env)
+                    }
+                }
             };
-            return res.map_err(|e| {
-                e.with_key_context(
-                    &self.key,
-                    Definition::Environment(self.key.as_env_key().to_string()),
-                )
-            });
+            return res.map_err(|e| e.with_key_context(&self.key, env_def));
         }
 
-        let o_cv = self.config.get_cv(self.key.as_config_key())?;
-        if let Some(cv) = o_cv {
-            let res: (Result<V::Value, ConfigError>, PathBuf) = match cv {
-                CV::Integer(i, path) => (visitor.visit_i64(i), path),
-                CV::String(s, path) => (visitor.visit_string(s), path),
-                CV::List(_, path) => (visitor.visit_seq(ConfigSeqAccess::new(self.clone())?), path),
-                CV::Table(_, path) => (
+        if let Some(cv) = cv {
+            let res: (Result<V::Value, ConfigError>, Definition) = match cv {
+                CV::Integer(i, def) => (visitor.visit_i64(i), def),
+                CV::String(s, def) => (visitor.visit_string(s), def),
+                CV::List(_, def) => (visitor.visit_seq(ConfigSeqAccess::new(self.clone())?), def),
+                CV::Table(_, def) => (
                     visitor.visit_map(ConfigMapAccess::new_map(self.clone())?),
-                    path,
+                    def,
                 ),
-                CV::Boolean(b, path) => (visitor.visit_bool(b), path),
+                CV::Boolean(b, def) => (visitor.visit_bool(b), def),
             };
-            let (res, path) = res;
-            return res.map_err(|e| e.with_key_context(&self.key, Definition::Path(path)));
+            let (res, def) = res;
+            return res.map_err(|e| e.with_key_context(&self.key, def));
         }
         Err(ConfigError::missing(&self.key))
     }
 
-    deserialize_method!(deserialize_bool, visit_bool, get_bool_priv);
+    deserialize_method!(deserialize_bool, visit_bool, get_bool);
     deserialize_method!(deserialize_i8, visit_i64, get_integer);
     deserialize_method!(deserialize_i16, visit_i64, get_integer);
     deserialize_method!(deserialize_i32, visit_i64, get_integer);
@@ -95,7 +118,7 @@ impl<'de, 'config> de::Deserializer<'de> for Deserializer<'config> {
     where
         V: de::Visitor<'de>,
     {
-        if self.config.has_key(&self.key) {
+        if self.config.has_key(&self.key, self.env_prefix_ok) {
             visitor.visit_some(self)
         } else {
             // Treat missing values as `None`.
@@ -117,7 +140,7 @@ impl<'de, 'config> de::Deserializer<'de> for Deserializer<'config> {
         //
         // See more comments in `value.rs` for the protocol used here.
         if name == value::NAME && fields == value::FIELDS {
-            return visitor.visit_map(ValueDeserializer { hits: 0, de: self });
+            return visitor.visit_map(ValueDeserializer::new(self)?);
         }
         visitor.visit_map(ConfigMapAccess::new_struct(self, fields)?)
     }
@@ -155,21 +178,40 @@ impl<'de, 'config> de::Deserializer<'de> for Deserializer<'config> {
         visitor.visit_seq(ConfigSeqAccess::new(self)?)
     }
 
+    fn deserialize_newtype_struct<V>(
+        self,
+        name: &'static str,
+        visitor: V,
+    ) -> Result<V::Value, Self::Error>
+    where
+        V: de::Visitor<'de>,
+    {
+        if name == "StringList" {
+            let vals = self.config.get_list_or_string(&self.key)?;
+            let vals: Vec<String> = vals.into_iter().map(|vd| vd.0).collect();
+            visitor.visit_newtype_struct(vals.into_deserializer())
+        } else {
+            visitor.visit_newtype_struct(self)
+        }
+    }
+
     // These aren't really supported, yet.
     serde::forward_to_deserialize_any! {
         f32 f64 char str bytes
         byte_buf unit unit_struct
-        enum identifier ignored_any newtype_struct
+        enum identifier ignored_any
     }
 }
 
 struct ConfigMapAccess<'config> {
     de: Deserializer<'config>,
-    set_iter: <HashSet<KeyKind> as IntoIterator>::IntoIter,
-    next: Option<KeyKind>,
+    /// The fields that this map should deserialize.
+    fields: Vec<KeyKind>,
+    /// Current field being deserialized.
+    field_index: usize,
 }
 
-#[derive(PartialEq, Eq, Hash)]
+#[derive(Debug, PartialEq, Eq, Hash)]
 enum KeyKind {
     Normal(String),
     CaseSensitive(String),
@@ -177,31 +219,31 @@ enum KeyKind {
 
 impl<'config> ConfigMapAccess<'config> {
     fn new_map(de: Deserializer<'config>) -> Result<ConfigMapAccess<'config>, ConfigError> {
-        let mut set = HashSet::new();
-        if let Some(mut v) = de.config.get_table(de.key.as_config_key())? {
+        let mut fields = Vec::new();
+        if let Some(mut v) = de.config.get_table(&de.key)? {
             // `v: Value<HashMap<String, CV>>`
             for (key, _value) in v.val.drain() {
-                set.insert(KeyKind::CaseSensitive(key));
+                fields.push(KeyKind::CaseSensitive(key));
             }
         }
         if de.config.cli_unstable().advanced_env {
-            // `CARGO_PROFILE_DEV_OVERRIDES_`
-            let env_pattern = format!("{}_", de.key.as_env_key());
+            // `CARGO_PROFILE_DEV_PACKAGE_`
+            let env_prefix = format!("{}_", de.key.as_env_key());
             for env_key in de.config.env.keys() {
-                if env_key.starts_with(&env_pattern) {
-                    // `CARGO_PROFILE_DEV_OVERRIDES_bar_OPT_LEVEL = 3`
-                    let rest = &env_key[env_pattern.len()..];
+                if env_key.starts_with(&env_prefix) {
+                    // `CARGO_PROFILE_DEV_PACKAGE_bar_OPT_LEVEL = 3`
+                    let rest = &env_key[env_prefix.len()..];
                     // `rest = bar_OPT_LEVEL`
                     let part = rest.splitn(2, '_').next().unwrap();
                     // `part = "bar"`
-                    set.insert(KeyKind::CaseSensitive(part.to_string()));
+                    fields.push(KeyKind::CaseSensitive(part.to_string()));
                 }
             }
         }
         Ok(ConfigMapAccess {
             de,
-            set_iter: set.into_iter(),
-            next: None,
+            fields,
+            field_index: 0,
         })
     }
 
@@ -209,33 +251,36 @@ impl<'config> ConfigMapAccess<'config> {
         de: Deserializer<'config>,
         fields: &'static [&'static str],
     ) -> Result<ConfigMapAccess<'config>, ConfigError> {
-        let mut set = HashSet::new();
-        for field in fields {
-            set.insert(KeyKind::Normal(field.to_string()));
-        }
+        let fields: Vec<KeyKind> = fields
+            .iter()
+            .map(|field| KeyKind::Normal(field.to_string()))
+            .collect();
 
         // Assume that if we're deserializing a struct it exhaustively lists all
         // possible fields on this key that we're *supposed* to use, so take
         // this opportunity to warn about any keys that aren't recognized as
         // fields and warn about them.
-        if let Some(mut v) = de.config.get_table(de.key.as_config_key())? {
+        if let Some(mut v) = de.config.get_table(&de.key)? {
             for (t_key, value) in v.val.drain() {
-                if set.contains(&KeyKind::Normal(t_key.to_string())) {
+                if fields.iter().any(|k| match k {
+                    KeyKind::Normal(s) => s == &t_key,
+                    KeyKind::CaseSensitive(s) => s == &t_key,
+                }) {
                     continue;
                 }
                 de.config.shell().warn(format!(
-                    "unused key `{}.{}` in config file `{}`",
-                    de.key.as_config_key(),
+                    "unused config key `{}.{}` in `{}`",
+                    de.key,
                     t_key,
-                    value.definition_path().display()
+                    value.definition()
                 ))?;
             }
         }
 
         Ok(ConfigMapAccess {
             de,
-            set_iter: set.into_iter(),
-            next: None,
+            fields,
+            field_index: 0,
         })
     }
 }
@@ -247,30 +292,61 @@ impl<'de, 'config> de::MapAccess<'de> for ConfigMapAccess<'config> {
     where
         K: de::DeserializeSeed<'de>,
     {
-        match self.set_iter.next() {
-            Some(key) => {
-                let name = match &key {
-                    KeyKind::Normal(s) | KeyKind::CaseSensitive(s) => s.as_str(),
-                };
-                let result = seed.deserialize(name.into_deserializer()).map(Some);
-                self.next = Some(key);
-                result
-            }
-            None => Ok(None),
+        if self.field_index >= self.fields.len() {
+            return Ok(None);
         }
+        let field = match &self.fields[self.field_index] {
+            KeyKind::Normal(s) | KeyKind::CaseSensitive(s) => s.as_str(),
+        };
+        seed.deserialize(field.into_deserializer()).map(Some)
     }
 
     fn next_value_seed<V>(&mut self, seed: V) -> Result<V::Value, Self::Error>
     where
         V: de::DeserializeSeed<'de>,
     {
-        match self.next.take().expect("next field missing") {
-            KeyKind::Normal(key) => self.de.key.push(&key),
-            KeyKind::CaseSensitive(key) => self.de.key.push_sensitive(&key),
-        }
+        let field = &self.fields[self.field_index];
+        self.field_index += 1;
+        // Set this as the current key in the deserializer.
+        let field = match field {
+            KeyKind::Normal(field) => {
+                self.de.key.push(field);
+                field
+            }
+            KeyKind::CaseSensitive(field) => {
+                self.de.key.push_sensitive(field);
+                field
+            }
+        };
+        // Env vars that are a prefix of another with a dash/underscore cannot
+        // be supported by our serde implementation, so check for them here.
+        // Example:
+        //     CARGO_BUILD_TARGET
+        //     CARGO_BUILD_TARGET_DIR
+        // or
+        //     CARGO_PROFILE_DEV_DEBUG
+        //     CARGO_PROFILE_DEV_DEBUG_ASSERTIONS
+        // The `deserialize_option` method does not know the type of the field.
+        // If the type is an Option<struct> (like
+        // `profile.dev.build-override`), then it needs to check for env vars
+        // starting with CARGO_FOO_BAR_. This is a problem for keys like
+        // CARGO_BUILD_TARGET because checking for a prefix would incorrectly
+        // match CARGO_BUILD_TARGET_DIR. `deserialize_option` would have no
+        // choice but to call `visit_some()` which would then fail if
+        // CARGO_BUILD_TARGET isn't set. So we check for these prefixes and
+        // disallow them here.
+        let env_prefix = format!("{}_", field).replace('-', "_");
+        let env_prefix_ok = !self.fields.iter().any(|field| {
+            let field = match field {
+                KeyKind::Normal(s) | KeyKind::CaseSensitive(s) => s.as_str(),
+            };
+            field.replace('-', "_").starts_with(&env_prefix)
+        });
+
         let result = seed.deserialize(Deserializer {
             config: self.de.config,
             key: self.de.key.clone(),
+            env_prefix_ok,
         });
         self.de.key.pop();
         result
@@ -284,45 +360,12 @@ struct ConfigSeqAccess {
 impl ConfigSeqAccess {
     fn new(de: Deserializer<'_>) -> Result<ConfigSeqAccess, ConfigError> {
         let mut res = Vec::new();
-        if let Some(v) = de.config.get_list(de.key.as_config_key())? {
-            for (s, path) in v.val {
-                res.push((s, Definition::Path(path)));
-            }
+        if let Some(v) = de.config._get_list(&de.key)? {
+            res.extend(v.val);
         }
 
-        if de.config.cli_unstable().advanced_env {
-            // Parse an environment string as a TOML array.
-            if let Some(v) = de.config.env.get(de.key.as_env_key()) {
-                let def = Definition::Environment(de.key.as_env_key().to_string());
-                if !(v.starts_with('[') && v.ends_with(']')) {
-                    return Err(ConfigError::new(
-                        format!("should have TOML list syntax, found `{}`", v),
-                        def,
-                    ));
-                }
-                let toml_s = format!("value={}", v);
-                let toml_v: toml::Value = toml::de::from_str(&toml_s).map_err(|e| {
-                    ConfigError::new(format!("could not parse TOML list: {}", e), def.clone())
-                })?;
-                let values = toml_v
-                    .as_table()
-                    .unwrap()
-                    .get("value")
-                    .unwrap()
-                    .as_array()
-                    .expect("env var was not array");
-                for value in values {
-                    // TODO: support other types.
-                    let s = value.as_str().ok_or_else(|| {
-                        ConfigError::new(
-                            format!("expected string, found {}", value.type_str()),
-                            def.clone(),
-                        )
-                    })?;
-                    res.push((s.to_string(), def.clone()));
-                }
-            }
-        }
+        de.config.get_env_list(&de.key, &mut res)?;
+
         Ok(ConfigSeqAccess {
             list_iter: res.into_iter(),
         })
@@ -353,9 +396,40 @@ impl<'de> de::SeqAccess<'de> for ConfigSeqAccess {
 /// See more comments in `value.rs` for the protocol used here.
 struct ValueDeserializer<'config> {
     hits: u32,
+    definition: Definition,
     de: Deserializer<'config>,
 }
 
+impl<'config> ValueDeserializer<'config> {
+    fn new(de: Deserializer<'config>) -> Result<ValueDeserializer<'config>, ConfigError> {
+        // Figure out where this key is defined.
+        let definition = {
+            let env = de.key.as_env_key();
+            let env_def = Definition::Environment(env.to_string());
+            match (de.config.env.contains_key(env), de.config.get_cv(&de.key)?) {
+                (true, Some(cv)) => {
+                    // Both, pick highest priority.
+                    if env_def.is_higher_priority(cv.definition()) {
+                        env_def
+                    } else {
+                        cv.definition().clone()
+                    }
+                }
+                (false, Some(cv)) => cv.definition().clone(),
+                // Assume it is an environment, even if the key is not set.
+                // This can happen for intermediate tables, like
+                // CARGO_FOO_BAR_* where `CARGO_FOO_BAR` is not set.
+                (_, None) => env_def,
+            }
+        };
+        Ok(ValueDeserializer {
+            hits: 0,
+            definition,
+            de,
+        })
+    }
+}
+
 impl<'de, 'config> de::MapAccess<'de> for ValueDeserializer<'config> {
     type Error = ConfigError;
 
@@ -379,31 +453,25 @@ impl<'de, 'config> de::MapAccess<'de> for ValueDeserializer<'config> {
     where
         V: de::DeserializeSeed<'de>,
     {
-        macro_rules! bail {
-            ($($t:tt)*) => (return Err(failure::format_err!($($t)*).into()))
-        }
-
         // If this is the first time around we deserialize the `value` field
         // which is the actual deserializer
         if self.hits == 1 {
-            return seed.deserialize(self.de.clone());
+            return seed
+                .deserialize(self.de.clone())
+                .map_err(|e| e.with_key_context(&self.de.key, self.definition.clone()));
         }
 
         // ... otherwise we're deserializing the `definition` field, so we need
         // to figure out where the field we just deserialized was defined at.
-        let env = self.de.key.as_env_key();
-        if self.de.config.env.contains_key(env) {
-            return seed.deserialize(Tuple2Deserializer(1i32, env));
+        match &self.definition {
+            Definition::Path(path) => {
+                seed.deserialize(Tuple2Deserializer(0i32, path.to_string_lossy()))
+            }
+            Definition::Environment(env) => {
+                seed.deserialize(Tuple2Deserializer(1i32, env.as_ref()))
+            }
+            Definition::Cli => seed.deserialize(Tuple2Deserializer(2i32, "")),
         }
-        let val = match self.de.config.get_cv(self.de.key.as_config_key())? {
-            Some(val) => val,
-            None => bail!("failed to find definition of `{}`", self.de.key),
-        };
-        let path = match val.definition_path().to_str() {
-            Some(s) => s,
-            None => bail!("failed to convert {:?} to utf-8", val.definition_path()),
-        };
-        seed.deserialize(Tuple2Deserializer(0i32, path))
     }
 }
 
