@@ -584,6 +584,10 @@ fn doc_same_name() {
 
 #[cargo_test]
 fn doc_target() {
+    if !is_nightly() {
+        // no_core, lang_items requires nightly.
+        return;
+    }
     const TARGET: &str = "arm-unknown-linux-gnueabihf";
 
     let p = project()
@@ -1014,7 +1018,7 @@ fn doc_all_member_dependency_same_name() {
 }
 
 #[cargo_test]
-#[cfg(not(any(target_os = "windows", target_os = "macos")))]
+#[cfg(not(windows))] // `echo` may not be available
 fn doc_workspace_open_help_message() {
     let p = project()
         .file(
@@ -1040,7 +1044,7 @@ fn doc_workspace_open_help_message() {
 }
 
 #[cargo_test]
-#[cfg(not(any(target_os = "windows", target_os = "macos")))]
+#[cfg(not(windows))] // `echo` may not be available
 fn doc_workspace_open_different_library_and_package_names() {
     let p = project()
         .file(
@@ -1071,7 +1075,7 @@ fn doc_workspace_open_different_library_and_package_names() {
 }
 
 #[cargo_test]
-#[cfg(not(any(target_os = "windows", target_os = "macos")))]
+#[cfg(not(windows))] // `echo` may not be available
 fn doc_workspace_open_binary() {
     let p = project()
         .file(
@@ -1103,7 +1107,7 @@ fn doc_workspace_open_binary() {
 }
 
 #[cargo_test]
-#[cfg(not(any(target_os = "windows", target_os = "macos")))]
+#[cfg(not(windows))] // `echo` may not be available
 fn doc_workspace_open_binary_and_library() {
     let p = project()
         .file(
@@ -1506,3 +1510,36 @@ fn bin_private_items_deps() {
     assert!(p.root().join("target/doc/bar/fn.bar_pub.html").is_file());
     assert!(!p.root().join("target/doc/bar/fn.bar_priv.html").exists());
 }
+
+#[cargo_test]
+fn crate_versions() {
+    // Testing unstable flag
+    if !is_nightly() {
+        return;
+    }
+    let p = project()
+        .file(
+            "Cargo.toml",
+            r#"
+            [package]
+            name = "foo"
+            version = "1.2.4"
+            authors = []
+        "#,
+        )
+        .file("src/lib.rs", "")
+        .build();
+
+    p.cargo("-Z crate-versions doc")
+        .masquerade_as_nightly_cargo()
+        .run();
+
+    let doc_file = p.root().join("target/doc/foo/index.html");
+    let mut doc_html = String::new();
+    File::open(&doc_file)
+        .unwrap()
+        .read_to_string(&mut doc_html)
+        .unwrap();
+
+    assert!(doc_html.contains("Version 1.2.4"));
+}
