@@ -2017,13 +2017,17 @@ fn virtual_ws_flags() {
         .build();
 
     p.cargo("build --features=f1")
-        .with_stderr("[ERROR] --features is not allowed in the root of a virtual workspace")
+        .with_stderr(
+            "[ERROR] --features is not allowed in the root of a virtual workspace\n\
+             note: while this was previously accepted, it didn't actually do anything",
+        )
         .with_status(101)
         .run();
 
     p.cargo("build --no-default-features")
         .with_stderr(
-            "[ERROR] --no-default-features is not allowed in the root of a virtual workspace",
+            "[ERROR] --no-default-features is not allowed in the root of a virtual workspace\n\
+             note: while this was previously accepted, it didn't actually do anything",
         )
         .with_status(101)
         .run();
@@ -2138,3 +2142,53 @@ fn all_features_virtual_ws() {
         .with_stdout("f1\nf2\nf3\n")
         .run();
 }
+
+#[cargo_test]
+fn slash_optional_enables() {
+    // --features dep/feat will enable `dep` and set its feature.
+    let p = project()
+        .file(
+            "Cargo.toml",
+            r#"
+            [package]
+            name = "foo"
+            version = "0.1.0"
+
+            [dependencies]
+            dep = {path="dep", optional=true}
+            "#,
+        )
+        .file(
+            "src/lib.rs",
+            r#"
+            #[cfg(not(feature="dep"))]
+            compile_error!("dep not set");
+            "#,
+        )
+        .file(
+            "dep/Cargo.toml",
+            r#"
+            [package]
+            name = "dep"
+            version = "0.1.0"
+
+            [features]
+            feat = []
+            "#,
+        )
+        .file(
+            "dep/src/lib.rs",
+            r#"
+            #[cfg(not(feature="feat"))]
+            compile_error!("feat not set");
+            "#,
+        )
+        .build();
+
+    p.cargo("check")
+        .with_status(101)
+        .with_stderr_contains("[..]dep not set[..]")
+        .run();
+
+    p.cargo("check --features dep/feat").run();
+}
