@@ -1,13 +1,14 @@
-use crate::support;
+//! Tests for the `cargo init` command.
+
 use std::env;
 use std::fs::{self, File};
 use std::io::prelude::*;
 use std::process::Command;
 
-use crate::support::{paths, Execs};
+use cargo_test_support::{command_is_available, paths, Execs};
 
 fn cargo_process(s: &str) -> Execs {
-    let mut execs = support::cargo_process(s);
+    let mut execs = cargo_test_support::cargo_process(s);
     execs.cwd(&paths::root()).env("HOME", &paths::home());
     execs
 }
@@ -59,7 +60,7 @@ fn simple_bin() {
 
 #[cargo_test]
 fn simple_git_ignore_exists() {
-    // write a .gitignore file with one entry
+    // write a .gitignore file with two entries
     fs::create_dir_all(paths::root().join("foo")).unwrap();
     fs::write(
         paths::root().join("foo/.gitignore"),
@@ -89,16 +90,41 @@ fn simple_git_ignore_exists() {
          **/some.file\n\n\
          #Added by cargo\n\
          #\n\
-         #already existing elements are commented out\n\
+         #already existing elements were commented out\n\
          \n\
          #/target\n\
-         **/*.rs.bk\n\
          Cargo.lock\n",
     );
 
     cargo_process("build").cwd(&paths::root().join("foo")).run();
 }
 
+#[cargo_test]
+fn git_ignore_exists_no_conflicting_entries() {
+    // write a .gitignore file with one entry
+    fs::create_dir_all(paths::root().join("foo")).unwrap();
+    fs::write(paths::root().join("foo/.gitignore"), "**/some.file").unwrap();
+
+    cargo_process("init --lib foo --edition 2015")
+        .env("USER", "foo")
+        .run();
+
+    let fp = paths::root().join("foo/.gitignore");
+    let mut contents = String::new();
+    File::open(&fp)
+        .unwrap()
+        .read_to_string(&mut contents)
+        .unwrap();
+    assert_eq!(
+        contents,
+        "**/some.file\n\n\
+         #Added by cargo\n\
+         \n\
+         /target\n\
+         Cargo.lock\n",
+    );
+}
+
 #[cargo_test]
 fn both_lib_and_bin() {
     cargo_process("init --lib --bin")
@@ -630,3 +656,50 @@ fn no_filename() {
         )
         .run();
 }
+
+#[cargo_test]
+fn formats_source() {
+    if !command_is_available("rustfmt") {
+        return;
+    }
+
+    fs::write(&paths::root().join("rustfmt.toml"), "tab_spaces = 2").unwrap();
+
+    cargo_process("init --lib")
+        .env("USER", "foo")
+        .with_stderr("[CREATED] library package")
+        .run();
+
+    assert_eq!(
+        fs::read_to_string(paths::root().join("src/lib.rs")).unwrap(),
+        r#"#[cfg(test)]
+mod tests {
+  #[test]
+  fn it_works() {
+    assert_eq!(2 + 2, 4);
+  }
+}
+"#
+    );
+}
+
+#[cargo_test]
+fn ignores_failure_to_format_source() {
+    cargo_process("init --lib")
+        .env("USER", "foo")
+        .env("PATH", "") // pretend that `rustfmt` is missing
+        .with_stderr("[CREATED] library package")
+        .run();
+
+    assert_eq!(
+        fs::read_to_string(paths::root().join("src/lib.rs")).unwrap(),
+        r#"#[cfg(test)]
+mod tests {
+    #[test]
+    fn it_works() {
+        assert_eq!(2 + 2, 4);
+    }
+}
+"#
+    );
+}
