@@ -1,11 +1,13 @@
-use crate::support::{
+//! Tests for the metabuild feature (declarative build scripts).
+
+use cargo_test_support::{
     basic_lib_manifest, basic_manifest, is_coarse_mtime, project, registry::Package, rustc_host,
     Project,
 };
-use serde_json;
+
 use std::str;
 
-#[test]
+#[cargo_test]
 fn metabuild_gated() {
     let p = project()
         .file(
@@ -73,7 +75,7 @@ fn basic_project() -> Project {
         .build()
 }
 
-#[test]
+#[cargo_test]
 fn metabuild_basic() {
     let p = basic_project();
     p.cargo("build -vv")
@@ -83,7 +85,7 @@ fn metabuild_basic() {
         .run();
 }
 
-#[test]
+#[cargo_test]
 fn metabuild_error_both() {
     let p = project()
         .file(
@@ -122,7 +124,7 @@ Caused by:
         .run();
 }
 
-#[test]
+#[cargo_test]
 fn metabuild_missing_dep() {
     let p = project()
         .file(
@@ -151,7 +153,7 @@ Caused by:
         .run();
 }
 
-#[test]
+#[cargo_test]
 fn metabuild_optional_dep() {
     let p = project()
         .file(
@@ -186,7 +188,7 @@ fn metabuild_optional_dep() {
         .run();
 }
 
-#[test]
+#[cargo_test]
 fn metabuild_lib_name() {
     // Test when setting `name` on [lib].
     let p = project()
@@ -226,7 +228,7 @@ fn metabuild_lib_name() {
         .run();
 }
 
-#[test]
+#[cargo_test]
 fn metabuild_fresh() {
     if is_coarse_mtime() {
         // This test doesn't work on coarse mtimes very well. Because the
@@ -277,7 +279,7 @@ fn metabuild_fresh() {
         .run();
 }
 
-#[test]
+#[cargo_test]
 fn metabuild_links() {
     let p = project()
         .file(
@@ -312,7 +314,7 @@ fn metabuild_links() {
         .run();
 }
 
-#[test]
+#[cargo_test]
 fn metabuild_override() {
     let p = project()
         .file(
@@ -350,7 +352,7 @@ fn metabuild_override() {
     p.cargo("build -vv").masquerade_as_nightly_cargo().run();
 }
 
-#[test]
+#[cargo_test]
 fn metabuild_workspace() {
     let p = project()
         .file(
@@ -409,7 +411,7 @@ fn metabuild_workspace() {
         )
         .build();
 
-    p.cargo("build -vv --all")
+    p.cargo("build -vv --workspace")
         .masquerade_as_nightly_cargo()
         .with_stdout_contains("[member1 0.0.1] Hello mb1 [..]member1")
         .with_stdout_contains("[member1 0.0.1] Hello mb2 [..]member1")
@@ -418,7 +420,7 @@ fn metabuild_workspace() {
         .run();
 }
 
-#[test]
+#[cargo_test]
 fn metabuild_metadata() {
     // The metabuild Target is filtered out of the `metadata` results.
     let p = basic_project();
@@ -444,7 +446,7 @@ fn metabuild_metadata() {
     assert_eq!(mb_info, ["mb", "mb-other"]);
 }
 
-#[test]
+#[cargo_test]
 fn metabuild_build_plan() {
     let p = basic_project();
 
@@ -461,7 +463,10 @@ fn metabuild_build_plan() {
             "compile_mode": "build",
             "kind": "Host",
             "deps": [],
-            "outputs": ["[..]/target/debug/deps/libmb-[..].rlib"],
+            "outputs": [
+                "[..]/target/debug/deps/libmb-[..].rlib",
+                "[..]/target/debug/deps/libmb-[..].rmeta"
+            ],
             "links": {},
             "program": "rustc",
             "args": "{...}",
@@ -475,7 +480,10 @@ fn metabuild_build_plan() {
             "compile_mode": "build",
             "kind": "Host",
             "deps": [],
-            "outputs": ["[..]/target/debug/deps/libmb_other-[..].rlib"],
+            "outputs": [
+                "[..]/target/debug/deps/libmb_other-[..].rlib",
+                "[..]/target/debug/deps/libmb_other-[..].rmeta"
+            ],
             "links": {},
             "program": "rustc",
             "args": "{...}",
@@ -517,7 +525,10 @@ fn metabuild_build_plan() {
             "compile_mode": "build",
             "kind": "Host",
             "deps": [3],
-            "outputs": ["[..]/foo/target/debug/deps/libfoo-[..].rlib"],
+            "outputs": [
+                "[..]/foo/target/debug/deps/libfoo-[..].rlib",
+                "[..]/foo/target/debug/deps/libfoo-[..].rmeta"
+            ],
             "links": "{...}",
             "program": "rustc",
             "args": "{...}",
@@ -538,7 +549,7 @@ fn metabuild_build_plan() {
     assert_eq!(p.glob("target/.metabuild/metabuild-foo-*.rs").count(), 1);
 }
 
-#[test]
+#[cargo_test]
 fn metabuild_two_versions() {
     // Two versions of a metabuild dep with the same name.
     let p = project()
@@ -604,7 +615,7 @@ fn metabuild_two_versions() {
         )
         .build();
 
-    p.cargo("build -vv --all")
+    p.cargo("build -vv --workspace")
         .masquerade_as_nightly_cargo()
         .with_stdout_contains("[member1 0.0.1] Hello mb1 [..]member1")
         .with_stdout_contains("[member2 0.0.1] Hello mb2 [..]member2")
@@ -616,7 +627,7 @@ fn metabuild_two_versions() {
     );
 }
 
-#[test]
+#[cargo_test]
 fn metabuild_external_dependency() {
     Package::new("mb", "1.0.0")
         .file("Cargo.toml", &basic_manifest("mb", "1.0.0"))
@@ -665,7 +676,7 @@ fn metabuild_external_dependency() {
     assert_eq!(p.glob("target/.metabuild/metabuild-dep-*.rs").count(), 1);
 }
 
-#[test]
+#[cargo_test]
 fn metabuild_json_artifact() {
     let p = basic_project();
     p.cargo("build --message-format=json")
@@ -686,6 +697,7 @@ fn metabuild_json_artifact() {
     "crate_types": [
       "bin"
     ],
+    "doctest": false,
     "edition": "2018",
     "kind": [
       "custom-build"
@@ -701,6 +713,7 @@ fn metabuild_json_artifact() {
   "linked_libs": [],
   "linked_paths": [],
   "package_id": "foo [..]",
+  "out_dir": "[..]",
   "reason": "build-script-executed"
 }
 "#,
@@ -708,7 +721,7 @@ fn metabuild_json_artifact() {
         .run();
 }
 
-#[test]
+#[cargo_test]
 fn metabuild_failed_build_json() {
     let p = basic_project();
     // Modify the metabuild dep so that it fails to compile.
@@ -723,7 +736,7 @@ fn metabuild_failed_build_json() {
     "children": "{...}",
     "code": "{...}",
     "level": "error",
-    "message": "cannot find function `metabuild` in module `mb`",
+    "message": "cannot find function `metabuild` in [..] `mb`",
     "rendered": "[..]",
     "spans": "{...}"
   },
@@ -733,6 +746,7 @@ fn metabuild_failed_build_json() {
     "crate_types": [
       "bin"
     ],
+    "doctest": false,
     "edition": "2018",
     "kind": [
       "custom-build"
