@@ -423,3 +423,93 @@ fn no_warning_ws() {
         )
         .run();
 }
+
+#[cargo_test]
+fn build_override_shared() {
+    // A dependency with a build script that is shared with a build
+    // dependency, using different profile settings. That is:
+    //
+    // foo DEBUG=2
+    // ├── common DEBUG=2
+    // │   └── common Run build.rs DEBUG=2
+    // │       └── common build.rs DEBUG=0 (build_override)
+    // └── foo Run build.rs DEBUG=2
+    //     └── foo build.rs DEBUG=0 (build_override)
+    //         └── common DEBUG=0 (build_override)
+    //             └── common Run build.rs DEBUG=0 (build_override)
+    //                 └── common build.rs DEBUG=0 (build_override)
+    //
+    // The key part here is that `common` RunCustomBuild is run twice, once
+    // with DEBUG=2 (as a dependency of foo) and once with DEBUG=0 (as a
+    // build-dependency of foo's build script).
+    Package::new("common", "1.0.0")
+        .file(
+            "build.rs",
+            r#"
+            fn main() {
+                if std::env::var("DEBUG").unwrap() != "false" {
+                    println!("cargo:rustc-cfg=foo_debug");
+                } else {
+                    println!("cargo:rustc-cfg=foo_release");
+                }
+            }
+            "#,
+        )
+        .file(
+            "src/lib.rs",
+            r#"
+            pub fn foo() -> u32 {
+                if cfg!(foo_debug) {
+                    assert!(cfg!(debug_assertions));
+                    1
+                } else if cfg!(foo_release) {
+                    assert!(!cfg!(debug_assertions));
+                    2
+                } else {
+                    panic!("not set");
+                }
+            }
+            "#,
+        )
+        .publish();
+
+    let p = project()
+        .file(
+            "Cargo.toml",
+            r#"
+            [package]
+            name = "foo"
+            version = "0.1.0"
+            edition = "2018"
+
+            [build-dependencies]
+            common = "1.0"
+
+            [dependencies]
+            common = "1.0"
+
+            [profile.dev.build-override]
+            debug = 0
+            debug-assertions = false
+            "#,
+        )
+        .file(
+            "build.rs",
+            r#"
+            fn main() {
+                assert_eq!(common::foo(), 2);
+            }
+            "#,
+        )
+        .file(
+            "src/main.rs",
+            r#"
+            fn main() {
+                assert_eq!(common::foo(), 1);
+            }
+            "#,
+        )
+        .build();
+
+    p.cargo("run").run();
+}
